//
//  PTAutomatorAdditions.m
//  Protein
//
//  Created by Quentin Carnicelli on 5/21/05.
//  Copyright 2005 Rogue Amoeba Software, LLC. All rights reserved.
//

#import "PTAutomatorAdditions.h"

@interface NSAppleEventDescriptor (ProteinAdditions)
+ (NSAppleEventDescriptor*)fsrefDescriptorFromFilePath: (NSString*)path;
+ (NSAppleEventDescriptor*)listOfFSRefsFromFilePaths: (NSArray*)filePaths;
@end

@interface NSObject (AutomatorFrameworkAdditions)
- (id)runWorkflowAtURL:(NSURL*)url withInput:(id)input error:(NSError**)error;
@end

@interface PTAutomatorWorkflow (MacOSX105Support)
- (NSError*)_105executeWithFiles: (NSArray*)filePaths;
@end

@interface PTAutomatorWorkflow (MacOSX104Support)
- (NSError*)_104executeWithFiles: (NSArray*)filePaths;
@end

@implementation PTAutomatorWorkflow

+ (BOOL)automatorSupported
{
	return floor(NSAppKitVersionNumber) > 743;
}

+ (BOOL)isWorkflowFile: (NSString*)path
{
	BOOL isDir;

	if( ![path length] )
		return NO;

	if( [[path pathExtension] caseInsensitiveCompare: @"workflow"] != 0 )
		return NO;

	if( [[NSFileManager defaultManager] fileExistsAtPath: path isDirectory: &isDir] == NO )
		return NO;

	if( isDir == NO )
		return NO;
		
	return YES;
}

+ (NSError*)executeWorkflow: (NSString*)workflowPath withFiles: (NSArray*)filePaths
{
	PTAutomatorWorkflow* workflow;
	NSError* error;
	
	workflow = [[PTAutomatorWorkflow alloc] initWithWorkflowFile: workflowPath];
	error = [workflow executeWithFiles: filePaths];
	[workflow release];

	return error;
}

- (id)initWithWorkflowFile: (NSString*)workflowPath
{
	self = [super init];
	
	if( self )
		mWorkflowPath = [workflowPath copy];

	return self;
}

- (void)dealloc
{
	[mWorkflowPath release];
	[super dealloc];
}

- (NSError*)executeWithFiles: (NSArray*)filePaths
{
	if( floor(NSAppKitVersionNumber) > 824 /*NSAppKitVersionNumber10_4*/ )
		return [self _105executeWithFiles: filePaths];
	return [self _104executeWithFiles: filePaths];
}

@end

#pragma mark -

@implementation PTAutomatorWorkflow (MacOSX105Support)

- (NSError*)_105executeWithFiles: (NSArray*)filePaths
{
	Class AMWorkflowClass = nil;
	NSError* error = nil;
	id results = nil;
		
	AMWorkflowClass = NSClassFromString(@"AMWorkflow");

	if( AMWorkflowClass == nil )
	{
		[[NSBundle bundleWithPath: @"/System/Library/Frameworks/Automator.framework/"] load];
		AMWorkflowClass = NSClassFromString(@"AMWorkflow");
		if( AMWorkflowClass == nil )
			return [NSError errorWithDomain: NSCocoaErrorDomain code: -1 userInfo: nil]; //Not really correct but good enough
	}

	
	results = [AMWorkflowClass runWorkflowAtURL: [NSURL fileURLWithPath: mWorkflowPath]
												withInput: filePaths
												error: &error];

	return error;
}

@end

#pragma mark -

@implementation NSAppleEventDescriptor (ProteinAdditions)

+ (NSAppleEventDescriptor*)fsrefDescriptorFromFilePath: (NSString*)path
{
	NSURL* fileURL;
	FSRef fsRef;
	
	if( !path )
		return nil;
	
	fileURL = [NSURL fileURLWithPath: path];
	if( !fileURL )
		return nil;
	
	if( !CFURLGetFSRef( (CFURLRef)fileURL, &fsRef ) )
		return nil;

	return 	[NSAppleEventDescriptor descriptorWithDescriptorType: typeFSRef bytes: &fsRef length: sizeof(fsRef)];
}

+ (NSAppleEventDescriptor*)listOfFSRefsFromFilePaths: (NSArray*)filePaths
{
	NSAppleEventDescriptor* fileList;
	NSAppleEventDescriptor* file;
	NSEnumerator* filePathsEnum;
	NSString* filePath;
	
	fileList = [NSAppleEventDescriptor listDescriptor]; 
	filePathsEnum = [filePaths objectEnumerator];
	
	while( (filePath = [filePathsEnum nextObject]) != nil )
	{
		file = [NSAppleEventDescriptor fsrefDescriptorFromFilePath: filePath];
		if( file )
			[fileList insertDescriptor: file atIndex: 0]; //This should be 0 or count+1, sayth the docs
	}
	
	return fileList;
}

@end

@implementation PTAutomatorWorkflow (MacOSX104Support)

- (NSURL*)_launchApplicationURL
{
	NSString* path;
	
	path = [[NSWorkspace sharedWorkspace] absolutePathForAppBundleWithIdentifier: @"com.apple.Automator_Launcher"];
	if( !path )
		path = @"/System/Library/CoreServices/Automator Launcher.app";
	
	return [NSURL fileURLWithPath: path];
}

- (NSAppleEventDescriptor*)_launchParamsDescriptor: (NSArray*)filePaths
{
	NSAppleEventDescriptor* paramRecord;
	NSAppleEventDescriptor* part;
	SInt32 cn;
	
	paramRecord = [NSAppleEventDescriptor recordDescriptor];
	
	//AMcn - I don't know what this setting is for, but if you set it to anything less then 1999 it doesnt work
	//       It seems to be related to the Finders Automator contextual menu (the first item is 1999, second is 2000, and so on).
	cn = 1999;
	part = [NSAppleEventDescriptor descriptorWithDescriptorType: typeSInt32 bytes: &cn length: sizeof(cn)];
	[paramRecord setDescriptor: part forKeyword: 'AMcn'];

	//AMap - This seems to be the calling application, but it also appears to be optional
	part = [NSAppleEventDescriptor fsrefDescriptorFromFilePath: [[[ProteinOwner sharedOwner] mainBundle] bundlePath]];
	[paramRecord setDescriptor: part forKeyword: 'AMap'];

	//AMsm - This is the actual workflow file
	part = [NSAppleEventDescriptor fsrefDescriptorFromFilePath: mWorkflowPath];
	[paramRecord setDescriptor: part forKeyword: 'AMsm'];

    //AMfs - This is a list of files to send to the workflow
	part = [NSAppleEventDescriptor listOfFSRefsFromFilePaths: filePaths];
	[paramRecord setDescriptor: part forKeyword: 'AMfs'];
	
	return paramRecord;
}

- (NSError*)_104executeWithFiles: (NSArray*)filePaths
{
	LSLaunchURLSpec spec;
	OSStatus err;

	spec.appURL			= (CFURLRef)[self _launchApplicationURL];
	spec.itemURLs		= nil;
	spec.passThruParams	= [[self _launchParamsDescriptor: filePaths] aeDesc];
	spec.launchFlags	= kLSLaunchDefaults;
	spec.asyncRefCon	= nil;

	err = LSOpenFromURLSpec( &spec, nil );
	if( err )
		return [NSError errorWithDomain: NSOSStatusErrorDomain code: err userInfo: nil];

	return nil;
}

@end
